/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include "emwin_support.h"
#include "peripherals.h"

#include "GUI.h"
#include "GUIDRV_Lin.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

AT_NONCACHEABLE_SECTION_ALIGN(uint8_t s_gui_memory[GUI_NUMBYTES * LCD_BYTES_PER_PIXEL], ELCDIF_1_RGB_BUFFER_ALIGN);

#define GUI_MEMORY_ADDR ((uint32_t)s_gui_memory)
#define VRAM_ADDR ((uint32_t)eLCDIF_1_Buffer[0])

static volatile int32_t s_LCDpendingBuffer = -1;
static SemaphoreHandle_t mutex;

extern void eLCDIF_1_init(void);

void ELCDIF_1_LCDIF_IRQHANDLER(void)
{
    uint32_t intStatus;

    intStatus = ELCDIF_GetInterruptStatus(ELCDIF_1_PERIPHERAL);

    ELCDIF_ClearInterruptStatus(ELCDIF_1_PERIPHERAL, intStatus);

    if (intStatus & kELCDIF_CurFrameDone)
    {
    	GUI_MULTIBUF_Confirm(s_LCDpendingBuffer);
    	s_LCDpendingBuffer = -1;
    }
}

void LCD_X_Config(void)
{
    GUI_MULTIBUF_Config(GUI_BUFFERS);
    GUI_DEVICE_CreateAndLink(DISPLAY_DRIVER, COLOR_CONVERSION, 0, 0);
    LCD_SetSizeEx(0, ELCDIF_1_PANEL_WIDTH, ELCDIF_1_PANEL_HEIGHT);
    LCD_SetVSizeEx(0, ELCDIF_1_PANEL_WIDTH, ELCDIF_1_PANEL_HEIGHT);
    LCD_SetVRAMAddrEx(0, (void *)VRAM_ADDR);
}

int LCD_X_DisplayDriver(unsigned LayerIndex, unsigned Cmd, void *p)
{
    uint32_t addr;
    int result = 0;
    LCD_X_SHOWBUFFER_INFO *pData;
    switch (Cmd)
    {
        case LCD_X_INITCONTROLLER:
        {
        	eLCDIF_1_init();
            break;
        }
        case LCD_X_SHOWBUFFER:
        {
            pData = (LCD_X_SHOWBUFFER_INFO *)p;
            addr = VRAM_ADDR + VRAM_SIZE * pData->Index;

            vTaskDelay(1);

            s_LCDpendingBuffer = pData->Index;
            ELCDIF_SetNextBufferAddr(ELCDIF_1_PERIPHERAL, addr);

            while (s_LCDpendingBuffer >= 0)
                ;
            return 0;
        }
        default:
            result = -1;
            break;
    }

    return result;
}

void GUI_X_Config(void)
{
    GUI_ALLOC_AssignMemory((void *)GUI_MEMORY_ADDR, GUI_NUMBYTES);
    GUI_SetDefaultFont(GUI_FONT_6X8);
}

void GUI_X_Init(void)
{
}

void GUI_X_InitOS(void)
{
    mutex = xSemaphoreCreateMutex();
}

void GUI_X_Lock(void)
{
    xSemaphoreTake(mutex, portMAX_DELAY);
}

void GUI_X_Unlock(void)
{
    xSemaphoreGive(mutex);
}

U32 GUI_X_GetTaskId(void)
{
    return 0;
}

void GUI_X_ExecIdle(void)
{
	vTaskDelay(1);
}

GUI_TIMER_TIME GUI_X_GetTime(void)
{
    return xTaskGetTickCount();
}

void GUI_X_Delay(int Period)
{
	vTaskDelay(Period);
}

void *emWin_memcpy(void *pDst, const void *pSrc, long size)
{
    return memcpy(pDst, pSrc, size);
}
