#include <stdio.h>
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"

#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"

#include "emwin_support.h"
#include "GUI.h"
#include "TEXT.h"
#include "BUTTON.h"
#include "DIALOG.h"

TaskHandle_t guiTaskHandle;

BUTTON_Handle button;

static uint32_t pressCounter = 0;

void eventCallback(WM_MESSAGE * pMsg)
{
	if(pMsg->MsgId == WM_NOTIFY_PARENT) {
		if (pMsg->hWinSrc == button) {
			int code = pMsg->Data.v;
			if (code == WM_NOTIFICATION_RELEASED) {
				pressCounter++;
			}
		}
		return;
	}

	WINDOW_Callback(pMsg);
}

void guiTask(void* param) {

	GUI_Init();
	WM_SetCreateFlags(WM_CF_MEMDEV);
	GUI_Clear();

	WM_HWIN window =  WINDOW_CreateEx(0, 0, LCD_GetXSize(), LCD_GetYSize(), WM_HBKWIN, WM_CF_SHOW, 0, GUI_ID_USER, NULL);
	WINDOW_SetBkColor(window,  GUI_BLACK);

	button = BUTTON_CreateEx(100, 140, 250, 200, window, WM_CF_SHOW, 0, GUI_ID_BUTTON0);
	BUTTON_SetText(button, "Press me!");
	BUTTON_SetFont(button, GUI_LARGE_FONT);

	char textBuffer[32];
	sprintf(textBuffer, "Press counter: %lu", pressCounter);
	TEXT_Handle text = TEXT_CreateEx(450, 215, 250, 50, window, WM_CF_SHOW, TEXT_CF_RIGHT, GUI_ID_TEXT0, textBuffer);
	TEXT_SetFont(text, GUI_LARGE_FONT);
	TEXT_SetTextColor(text, GUI_WHITE);

	WM_SetCallback(window, eventCallback);

    GUI_Exec();

    while(1)
    {
    	vTaskDelay(150 / portTICK_PERIOD_MS);
    	sprintf(textBuffer, "Press counter: %lu", pressCounter);
    	TEXT_SetText(text, textBuffer);
       	GUI_Exec();
    }
}

void touchTimer(TimerHandle_t xTimer)
{
	uint8_t data[5] = {0};

	flexio_i2c_master_transfer_t touchTransfer;
	touchTransfer.data = data;
	touchTransfer.dataSize = 5;
	touchTransfer.flags = 0;
	touchTransfer.slaveAddress = 0x38;
	touchTransfer.subaddressSize = 0x01;
	touchTransfer.subaddress = 0x02;
	touchTransfer.direction = kFLEXIO_I2C_Read;
	FLEXIO_I2C_MasterTransferBlocking(&FlexIO_I2C_1_peripheralConfig, &touchTransfer);

	uint8_t points = data[0];
	uint32_t x = ((data[1] & 0x0F) << 8) | (data[2]);
	uint32_t y = ((data[3] & 0x0F) << 8) | (data[4]);

	static bool activeTouch = false;

	GUI_PID_STATE state;
	state.Layer = 0;
	state.Pressed = 0;
	state.x = x;
	state.y = y;

	if (points == 1) {
		activeTouch = true;
		state.Pressed = 1;
		GUI_PID_StoreState(&state);
	} else if (activeTouch) {
		activeTouch = false;
		state.Pressed = 0;
		GUI_PID_StoreState(&state);
	}
}

void main(void) {
    BOARD_InitBootPins();
    BOARD_InitBootClocks();
    BOARD_InitBootPeripherals();

    xTaskCreate(guiTask, "GUI", 1024, NULL, 1, &guiTaskHandle);

    TimerHandle_t timer = xTimerCreate("TOUCH", 150 / portTICK_PERIOD_MS, pdTRUE, NULL, touchTimer);
    xTimerStart(timer, 0);

    vTaskStartScheduler();
}
